part of '../flutterx_live_data.dart';

abstract class SelfLiveData<T extends SelfLiveData<T>> extends LiveData<T> {
  final Set<LiveData> _sources = {};

  SelfLiveData() {
    super.setData(this as T);
  }

  @override
  @nonVirtual
  bool shouldUpdateValue(T oldValue, T newValue) => false;

  @override
  void setData(T value) => throw UnsupportedError('cannot set this instance');

  void setSources(Set<LiveData> sources) {
    for (final source in _sources.toSet()) if (!sources.contains(source)) removeSource(source);
    sources.forEach(addSource);
  }

  void addSource(LiveData source) {
    if (_sources.add(source) && isActive) source.observeForever(_invalidate);
  }

  void removeSource(LiveData source) {
    if (_sources.remove(source) && isActive) source.removeObserver(_invalidate);
  }

  @override
  void onActive() {
    super.onActive();
    for (final source in _sources) source.observeForever(_invalidate);
  }

  @override
  void onInactive() {
    for (final source in _sources) source.removeObserver(_invalidate);
    super.onInactive();
  }

  void _invalidate(dynamic _) => invalidate();
}
