part of '../flutterx_live_data.dart';

typedef Observer<T> = void Function(T value);

class ObserverWrapper<T> {
  @protected
  final LiveData<T> data;
  @protected
  final Observer<T> onChanged;
  @protected
  final Dispatcher? dispatcher;

  bool _active = false;

  bool get active => _active;

  int _lastVersion = _uninitialized;

  ObserverWrapper(this.data, this.onChanged, this.dispatcher);

  @protected
  @mustCallSuper
  void onAdded() => _changeActiveState(active: true);

  @protected
  @mustCallSuper
  void onRemove() => _changeActiveState(active: false);

  bool updated(LiveData data, Object onChanged) => identical(this.data, data) && this.onChanged == onChanged;

  void dispose() => data.removeObserver(onChanged);

  void _changeActiveState({required bool active}) {
    if (_active == active) return;
    _active = active;
    data._changeActiveState(this, active: active);
  }

  late T _pendingData;
  bool _dispatchingPendingData = false;

  void _considerNotify(T data, int version) {
    if (_active && version > _lastVersion) {
      _lastVersion = version;
      final dispatcher = this.dispatcher;
      if (dispatcher == null) return onChanged(data);
      _pendingData = data;
      if (_dispatchingPendingData) return;
      _dispatchingPendingData = true;
      dispatcher.post(() {
        if (_active) onChanged(_pendingData);
        _dispatchingPendingData = false;
      });
    }
  }
}
